# Python code to cat patch files for Forties
#   - currently determines the patches based on teh data itself
#   - location of file locations are hard-coded
#   - contains extra input for patch number
#   - primarily for reproducibility
#
# Jason Chang
# July 2015

#----------------------------------------
# import modules
#----------------------------------------
import sys
import commands
import random
import re


#----------------------------------------
# script parameters
#----------------------------------------
tmp_num = random.randint(0,10000)


#----------------------------------------
# parse command line
#----------------------------------------
def parse_commandline():
  # check number of arguments
  if (len(sys.argv) < 1):
    print "Usage:"
    print "	python %s [patch number] [files to cat]" %(sys.argv[0])
    sys.exit()

  # grab files
  pnum  = int(sys.argv[1])
  files = []
  count = 1
  while 1:
    # counter
    count = count+1

    # determine whether argument exists or not
    try:
      files.append(sys.argv[count])
    except:
      break

  # clean up
  return pnum,files


#----------------------------------------
# cat files in order
#----------------------------------------
def run_cat_files(files,pnum):
  # allocate
  pdict = {}

  # determine the limits of the files
  for file in files:
    # grab start and end patches
    o2_com = """Get < %s o2 parform=n""" %(file)
    print o2_com
    o2 = float(commands.getoutput(o2_com))
    n2_com = """Get < %s n2 parform=n""" %(file)
    print n2_com
    n2 = int(commands.getoutput(n2_com))
    pdict['%s' %(file)] = (o2,o2+n2)

  # determine beginning and end patch indices
  # NOTE: will need to switch min/max to match goal of "files"
  pstart = int(min([x[0] for x in pdict.values()]))
  pend   = int(max([x[1] for x in pdict.values()]))

  # grab information for output name
  x = re.compile("^.+_(\w)_APAC_(\d\d)_\d+_(\w)\.H$")
  f = x.search(file)
  if f:
    pre = f.group(1)
    loc = f.group(2)
    dir = f.group(3)

  # determine if patch number makes sense
  if (pnum < pstart or pnum > pend):
    print "Patch number is out of range"
    sys.exit()

  # create patch
  for ipatch in range(pnum,pnum+1):
    # reset list and name of output file
    catlist = []
    outfile = "%s_APAC_%s_%s_%s.H" %(pre,loc,dir,str(ipatch).zfill(4))

    # loop over each file
    for file in files:
      # check if file should be included
      fstart = pdict['%s' %(file)][0]
      fend   = pdict['%s' %(file)][1]
      if ((fstart > pnum) or (fend < pnum)):
        continue

      # name of temp file
      tmpfile = "tmp_%s" %(file.split('/')[-1])
      catlist.append(tmpfile)

      # calculate and window out the appropriate patch
      o2 = pdict['%s' %(file)][0]
      f2 = ipatch-o2
      wind_com = """Window3d < %s n2=1 f2=%d synch=1 > %s""" %(file,f2,tmpfile)
      print wind_com
      commands.getoutput(wind_com)

    # cat data together
    cat_files(catlist,outfile)


#----------------------------------------
# function for catting files
#----------------------------------------
def cat_files(fnames,target):
  block=20
  nfiles=len(fnames)

  count2=0
  while 1:
    total=len(fnames)
    cnames = []

    count1=0
    if (total//block>1):
      # For all complete blocks
      for ii in range(total//block):
        names = fnames[(count1)*block:(count1+1)*block]
        cname = 'temp%s_%i.H' %(str(count2),tmp_num)
        print "Cat3d %s axis=2 > %s" %(" ".join(names),cname)
        print commands.getstatusoutput("Cat3d max_memory=1000 %s axis=2 > %s" %(" ".join(names),cname))
        cnames.append(cname)
        for jj in names:
          commands.getstatusoutput("Rm %s %s@@"%(jj,jj))
        count1 = count1+1
        count2 = count2+1

      # For the remainder complete blocks
      if (total%block==1): # If only one file
        cname = fnames[count1*block]
        cnames.append(cname)

        count1 = count1+1
        count2 = count2+1

      if (total%block>1):
        names = fnames[count1*block:count1*block+total%block]
        cname = 'temp%s_%i.H' %(str(count2),tmp_num)
        print "Cat3d %s axis=2 > %s" %(" ".join(names),cname)
        print commands.getstatusoutput("Cat3d max_memory=1000 %s axis=2 > %s" %(" ".join(names),cname))
        cnames.append(cname)
        for jj in names:
          commands.getstatusoutput("Rm %s %s@@"%(jj,jj))
        count1 = count1+1
        count2 = count2+1

      # Repeat with new list
      fnames=cnames

    else:
      names = fnames
      print "Cat3d %s axis=2 > %s" %(" ".join(names),target)
      print commands.getstatusoutput("Cat3d max_memory=1000 %s axis=2 > %s" %(" ".join(names),target))
      cnames.append(target)
      for jj in names:
        commands.getstatusoutput("Rm %s %s@@"%(jj,jj))
      break



#----------------------------------------
# run commands
#----------------------------------------
pnum,files = parse_commandline()
run_cat_files(files,pnum)
