program jcExpandHeadersSI_forties
  ! Program to add more parameters to the header file AFTER correlations
  ! Tailored toward Forties data
  !
  ! Inputs:
  !   1) receivers header information
  !   2) sources header information
  !
  ! Output:
  !   1) a new header file for correlations

  use sep
  implicit none

  ! declare variables
  integer                             :: ii
  integer                             :: nr1,nr2,ns1,ns2,nkeys
  integer                             :: rx_key,ry_key,rz_key
  integer                             :: sx_key,sy_key,sz_key
  integer                             :: rx_ind,ry_ind,rz_ind
  integer                             :: sx_ind,sy_ind,sz_ind
  real,allocatable,dimension (:,:)    :: sheader,rheader,header

  ! open I/O
  call sep_init()
  
  if (exist_file('r_head')) then
    call from_aux('r_head','n1',nr1,1 )
    call from_aux('r_head','n2',nr2,1 )
  else
    call seperr('Need file containing receivers header info')
  end if

  if (exist_file('s_head')) then
    call from_aux('s_head','n1',ns1,1 )
    call from_aux('s_head','n2',ns2,1 )
  else
    call seperr('Need file containing sources header info')
  end if

  ! get the indices of the keys you care about
  call from_param('rx_ind',rx_ind,30)
  call from_param('ry_ind',ry_ind,31)
  call from_param('rz_ind',rz_ind, 6)
  call from_param('sx_ind',sx_ind,30)
  call from_param('sy_ind',sy_ind,31)
  call from_param('sz_ind',sz_ind, 6)

  ! calculate the number of keys you care about for output
  call from_param('nkeys' ,nkeys ,6)
  call from_param('rx_key',rx_key,1)
  call from_param('ry_key',ry_key,2)
  call from_param('rz_key',rz_key,3)
  call from_param('sx_key',sx_key,4)
  call from_param('sy_key',sy_key,5)
  call from_param('sz_key',sz_key,6)

  call to_history('n1',nkeys  )
  call to_history('n2',nr2)
  call to_history('n3',ns2)

  call sep_close()

  ! allocate
  allocate(rheader(nr1 ,nr2))
  allocate(sheader(ns1 ,ns2))
  allocate(header(nkeys,nr2))

  ! read in
  call sreed("r_head",rheader,4*nr1*nr2)
  call sreed("s_head",sheader,4*ns1*ns2)

  ! fill header with appropriate info
  do ii=1,ns2
    ! copy over header info
    header(rx_key     ,:) = rheader(rx_ind     ,: )
    header(ry_key     ,:) = rheader(ry_ind     ,: )
    header(rz_key     ,:) = rheader(rz_ind     ,: )
    header(sx_key     ,:) = sheader(sx_ind     ,ii)
    header(sy_key     ,:) = sheader(sy_ind     ,ii)
    header(sz_key     ,:) = sheader(sz_ind     ,ii)

    ! write out
    call srite('out',header,4*nkeys*nr2)
  end do

  ! clean up
  deallocate(rheader,sheader,header)


end program
