program jcSelect
  ! Jason Chang
  ! August 2012
  ! Revised: February 2013
  !
  ! Window out stations based on location
  ! Like srSparseSelect
  !
  ! Input:
  !   - history file (station number is index n2)
  !   - header file with station location
  !   - opt: 1-rectangle
  !          2-circle
  !   - kill paramters (nkill and kill indices)
  !   - threshold paramters (thresh val, range to search in ilo1:ihi1)
  !
  ! Output:
  !   - Windowed history file, hopefully smaller now

  use sep
  implicit none

  ! declare variables
  integer                           :: ii,jj,kk
  integer                           :: n1,n2,n3
  integer                           :: n1xy,n2xy,n3xy,xind,yind
  integer                           :: ntraces,nkill
  integer                           :: opt,ilo1,ihi1
  integer,allocatable,dimension (:) :: kill
  real                              :: o1,d1
  real                              :: thresh,lo1,hi1,maxi
  real                              :: xmin,ymin,xmax,ymax
  real                              :: r,cx,cy,unit
  real,allocatable,dimension (:)    :: in,header
  logical                           :: verb,keep

  ! open input
  call sep_init()

  call from_param('verb',verb,.false.)

  call from_history('n1',n1,1 )
  call from_history('o1',o1,1.)
  call from_history('d1',d1,1.)
  call from_history('n2',n2,1 )
  call from_history('n3',n3,1 )

  ! threshold parameters
  call from_param('thresh',thresh,0.                 )
  call from_param('lo1'   ,lo1   ,o1                 )
  call from_param('ilo1'  ,ilo1  ,nint((lo1-o1)/d1)+1)
  call from_param('hi1'   ,hi1   ,o1+(n1-1)*d1       )
  call from_param('ihi1'  ,ihi1  ,nint((hi1-o1)/d1)+1)

  ! area parameters
  call from_param('opt'  ,opt  ,1 )
  call from_param('unit' ,unit ,1.)

  ! kill parameters
  call from_param('nkill',nkill,0 )
  allocate(kill(nkill))
  if ( nkill.ge.1 ) call from_param('kill',kill,(/1/)) 

  ! parameters if rectangle
  if ( opt.eq.1 ) then
    call from_param('xmin',xmin,-1000000.)
    call from_param('xmax',xmax, 1000000.)
    call from_param('ymin',ymin,-1000000.)
    call from_param('ymax',ymax, 1000000.)
  end if
  ! parameters if circle
  if ( opt.eq.2 ) then
    call from_param('r' ,r ,10000000.)
    call from_param('cx',cx,0.       )
    call from_param('cy',cy,0.       )
  end if

  if (exist_file ('xy_in')) then
    call from_param('xind',xind,1)
    call from_param('yind',yind,2)
    call from_aux('xy_in','n1',n1xy  )
    call from_aux('xy_in','n2',n2xy  )
    call from_aux('xy_in','n3',n3xy,1)
    if ( n2 .ne. n2xy ) call erexit ("n2 and n2xy do not match")
    if ( n3 .ne. n3xy ) call erexit ("n3 and n3xy do not match")
  else
    call erexit("Need xy_in header file")
  end if

  ! allocate
  allocate(in(n1))
  allocate(header(n1xy))

  if ( opt.eq.1 ) then
    ! start looping over stations
    do jj=1,n3
      ntraces = 0
      do ii=1,n2
        keep = .true.
        in = 0.
        header = 0.

        ! call in data and header
        call sreed("in",in,4*n1)
        call sreed("xy_in",header,4*n1xy)

        ! kill check
        do kk=1,nkill
          if ( ii.eq.kill(kk) ) keep=.false.
        end do

        ! threshold check
        if ( thresh.ne.0. ) then
          maxi = maxval(in(ilo1:ihi1))
          if ( maxi.gt.thresh ) keep=.false.
        end if

        ! area check
        if ((header(xind)*unit .lt. xmin*unit) .and. (header(xind)*unit .gt. xmax*unit) .and. (header(yind)*unit .lt. ymin*unit) .and. (header(yind)*unit .gt. ymax*unit)) keep=.false.

        ! write out
        if (keep) then
          call srite("out",in,4*n1)
          if (exist_file('xy_out')) call srite("xy_out",header,4*n1xy)
          ntraces = ntraces + 1
        end if
      end do
    end do
  end if

  if ( opt.eq.2 ) then
    ! start looping over stations
    do jj=1,n3
      ntraces = 0
      do ii=1,n2
        keep = .true.
        in = 0.
        header = 0.

        ! call in data and header
        call sreed("in",in,4*n1)
        call sreed("xy_in",header,4*n1xy)

        ! kill check
        do kk=1,nkill
          if ( ii.eq.kill(kk) ) keep=.false.
        end do

        ! threshold check
        if ( thresh.ne.0. ) then
          maxi = maxval(in(ilo1:ihi1))
          if ( maxi.gt.thresh ) keep=.false.
        end if

        ! area check
        if (sqrt((header(xind)*unit-cx)**2.+(header(yind)*unit-cy)**2.).gt.r) keep=.false.

        ! write out
        if (keep) then
          call srite("out",in,4*n1)
          if (exist_file('xy_out')) call srite("xy_out",header,4*n1xy)
          ntraces = ntraces + 1
        end if
      end do
    end do
  end if

  ! open output
  call to_history('n1',n1)
  call to_history('n2',ntraces)
  call to_history('n3',n3)
  if ( exist_file('xy_out') ) then
    call to_history('n1',n1xy,'xy_out')
    call to_history('n2',ntraces,'xy_out')
    call to_history('n3',n3xy,'xy_out')
  end if

  call sep_close()

  deallocate(in,header,kill)

end program jcSelect
