program jcXY2cmplx
! Jason P. Chang
! June 2012
! Updates
!   June 2015: can handle integer header inputs
!
! convert two real values to a single complex number
! originally for creating a stations map in Grey
! need to window the x and y coordinates
!
! Order: order of x and y coordinates
!   - 1: x is first index, y is second index [default]
!   - 2: y is first index, x is second index
  
  use sep
  implicit none
  
  ! allocate variables
  integer                              :: order
  integer                              :: i
  integer                              :: n1,n2
  real                                 :: a,b
  complex,allocatable,dimension   (:)  :: out
  integer,allocatable,dimension (:,:)  :: ini
  real   ,allocatable,dimension (:,:)  :: inr
  logical                              :: real
  
  ! open I/O
  call sep_init()

  call from_history('n1'   ,n1     )
  call from_history('n2'   ,n2     )
  call from_param  ('order',order,1)
  call from_param  ('real' ,real,.true.)
  
  call to_history('n1',  n2)
  call to_history('n2',   1)
  call to_history('esize',8)

  call sep_close()

  ! allocate
  if (real) then
    allocate(inr(n1,n2))
  else
    allocate(ini(n1,n2))
  end if
  allocate(out(n2))

  ! read in the coordinates
  if (real) then
    call sreed('in',inr,4*n1*n2)
  else
    call sreed('in',ini,4*n1*n2)
  end if

  ! form the complex value
  if (real) then
    do i=1,n2
      if ( order .eq. 1 ) then
        out(i) = cmplx( inr(1,i), inr(2,i) )
      else
        out(i) = cmplx( inr(2,i), inr(1,i) )
      end if
    end do
  else
    do i=1,n2
      if ( order .eq. 1 ) then
        out(i) = cmplx( ini(1,i), ini(2,i) )
      else
        out(i) = cmplx( ini(2,i), ini(1,i) )
      end if
    end do
  end if

  ! write out result
  call sep_write(out)

  ! clean up
  deallocate(out)
  if (allocated(ini)) deallocate(ini)
  if (allocated(inr)) deallocate(inr)
  call exit(0)
  
end program jcXY2cmplx
