program srAverage
  ! Program to average data in double prescision,
  ! either in first or second direction.
  !
  !
  ! options:
  ! dir=1/2*  (2nd direction is default)
  ! squeeze=T*/F (squeezes the empty axis)
  ! norm=T*/F (normalize or not (F is a stack basically)
  
  use sep
  implicit none
  
  integer                                     :: n1,n2,n3,n4,n5,n6
  real                                        ::    d2,d3,d4,d5,d6
  real                                        ::    o2,o3,o4,o5,o6
  character*50                                ::    label2,label3,label4,label5,label6
  
  real, allocatable, dimension(:)             :: datain
  double precision                            :: mean1,scaler
  double precision, allocatable, dimension(:) :: mean2(:)
  integer                                     :: jj,kk 
  
  integer                                     :: dir
  logical                                     :: squeeze,norm, verbose
  
  ! Open I/O headerfiles
  call sep_init()
  call from_param('verbose',verbose,.false.)
  call from_param('verb'   ,verbose,verbose)
  if (verbose) write(0,*) '-----------------------------------'
  if (verbose) write(0,*) '    DOUBLE PRESCISION AVERAGING    '
  if (verbose) write(0,*) '-----------------------------------'
  if (verbose) write(0,*) '    i/o looping over traces        '
  
  call from_param  ("dir",dir,2)
  call from_param('squeeze',squeeze,.true.)
  if (dir.lt.1.or.dir.gt.2) dir=2
  
  call from_history("n1",n1,1 )
  call from_history("n2",n2,1 )
  call from_history("n3",n3,1 )
  call from_history("n4",n4,1 )
  call from_history("n5",n5,1 )
  call from_history("n6",n6,1 )
  
  call from_history("d2",d2,1.)
  call from_history("d3",d3,1.)
  call from_history("d4",d4,1.)
  call from_history("d5",d5,1.)
  call from_history("d6",d6,1.)
  
  call from_history("o2",o2,0.)
  call from_history("o3",o3,0.)
  call from_history("o4",o4,0.)
  call from_history("o5",o5,0.)
  call from_history("o6",o6,0.)
  
  call from_history("label2",label2," ")
  call from_history("label3",label3," ")
  call from_history("label4",label4," ")
  call from_history("label5",label5," ")
  call from_history("label6",label6," ")
  
  if (dir.eq.1 ) then
    if (squeeze) then
      call to_history("n1",n2)
      call to_history("n2",n3)
      call to_history("n3",n4)
      call to_history("n4",n5)
      call to_history("n5",n6)
      call to_history("n6", 1)
      call to_history("d1",d2)
      call to_history("d2",d3)
      call to_history("d3",d4)
      call to_history("d4",d5)
      call to_history("d5",d6)
      call to_history("d6",1.)
      call to_history("o1",o2)
      call to_history("o2",o3)
      call to_history("o3",o4)
      call to_history("o4",o5)
      call to_history("o5",o6)
      call to_history("o6",0.)
      call to_history("label1",label2)
      call to_history("label2",label3)
      call to_history("label3",label4)
      call to_history("label4",label5)
      call to_history("label5",label6)
      call to_history("label6"," ")
    else
      call to_history('n1',1 )
    end if
  end if
  if (dir.eq.2 ) then
    if (squeeze) then
      call to_history("n2",n3)
      call to_history("n3",n4)
      call to_history("n4",n5)
      call to_history("n5",n6)
      call to_history("n6", 1)
      call to_history("d2",d3)
      call to_history("d3",d4)
      call to_history("d4",d5)
      call to_history("d5",d6)
      call to_history("d6",1.)
      call to_history("o2",o3)
      call to_history("o3",o4)
      call to_history("o4",o5)
      call to_history("o5",o6)
      call to_history("o6",0.)
      call to_history("label2",label3)
      call to_history("label3",label4)
      call to_history("label4",label5)
      call to_history("label5",label6)
      call to_history("label6"," ")
    else
      call to_history('n2',1 )
    end if
  end if
  
  call sep_close()
  
  allocate(datain(n1))
  if (dir.eq.2) allocate(mean2(n1))

  if (norm) then
    if (dir.eq.1) scaler=1./dble(n1)
    if (dir.eq.2) scaler=1./dble(n2)
  else
    scaler=1.
  end if
  
  if (verbose) write(0,*) '-------------------------------------------'
  if (dir.eq.2) mean2=0.
  !!!! Actual Kernel
  do kk=1,n3*n4*n5*n6
    if (dir.eq.2) mean2=0.
    do jj=1,n2
      if (verbose) write(0,"(A,I7.1,A,I7.3)") 'trace: ',(kk-1)*n2+jj,' out of',n2*n3*n4*n5*n6
    
      datain = 0.
      if (dir.eq.1) mean1=0.
    
      ! Read the data
      call sreed('in',datain,4*n1)
    
      ! Compute Mean
      if (dir.eq.1) mean1 =  sum  ( dble(datain) )
      if (dir.eq.2) mean2 = mean2 + dble(datain)
    
      ! Write the data
      if (dir.eq.1) call srite('out',real(mean1*scaler),4)
    
    end do
    
    ! Write the data
    if (dir.eq.2) call srite('out',real(mean2*scaler),4*n1)
  end do

  ! Deallocate memory
  if (verbose) write(0,*) '-----------------------------------'
  deallocate(datain)
  if (dir.eq.2) deallocate(mean2)
  call exit(0)
  
end program
